<?php

/**
 * Description of OrderModel
 *
 * @author Marie Dorušková
 */

namespace App\Model;

use Nette,
    ChromePhp;

class OrderModel {

    private $db = null;

    public function __construct($db) {
        $this->db = $db;
    }

    /** Vrací všechny nahrávací technologie
     * @return array of objects  
     */
    public function getTechnologies() {
        $stmt = $this->db->query("SELECT technology_id, name "
                . "FROM recording_technologies");
        return $stmt->fetchAll();
    }
    
    /** Vrací email uživatele
     * @param int $id ID uživatele
     * @return object
     */
    public function getEmail($id){
        $stmt = $this->db->query("SELECT user_id, email "
                . "FROM users "
                . "WHERE user_id = ? ", $id);
        return $stmt->fetch();
    }

    /** Vrací všechny hudební nástroje a jejich počet
     * @return array of objects
     */
    public function getInstruments() {
        $stmt = $this->db->query("SELECT name, value, label, instrument_id "
                . "FROM instruments "
                . "JOIN instrument_counts USING(instrument_id)"
                . "ORDER BY value ASC");
        return $stmt->fetchAll();
    }

    /** Vrací vybrané služby
     * @return array of objects 
     */
    public function getServices() {
        $stmt = $this->db->query("SELECT service_id, service_name, label FROM services "
                . "WHERE service_name IN "
                . "('Conductor', 'ClickTrack', 'MusicPreparation',"
                . "'SoundEditing', 'Orchestration')"
                . "GROUP BY service_id;");
        return $stmt->fetchAll();
    }

    /** Vrací všechny objednávky se základními údaji 
     * @return array of objects
     */
    public function getOrders($id) {
        $stmt = $this->db->query("SELECT users.user_id, users.name, users.surname, order_id, 
            order_date, order_status, record_term, total_price 
            FROM orders 
            JOIN users USING (user_id) 
            WHERE user_id = ?", $id);
        return $stmt->fetchAll();
    }
    

    /** Výpočet objednávky dle následujícího vzorce: 
     * Cena za hudebníka (A)
       Cena za dirigenta (B)
       Cena za studio (C)
       Profit (D)
       Marže obchodníka za platbu kartou (E)
       DPH (F)
       [(Počet hudebníků x A)+B+C+D]x počet hodin =  suma * platba kartou (v %) + DPH (21%). 
     * @param array $values 
     * @param int $userId ID uživatele
     */
    public function calculateOrder($values, $userId) {
        try {

            $dateCreate = date_create($values['date']);
            $date = date_format($dateCreate, "Y-m-d");

            //počet hodin nahrávání
            $hours = $values['hours'];

            //Pole instrumentů
            $hodnoty = array();
            $instrumentStmt = $this->db->query("SELECT name FROM instruments");
            $instrumenty = $instrumentStmt->fetchAll();

            foreach ($instrumenty as $ins) {

                $hodnoty[$ins['name']] = $values[$ins['name']];
            }
           // ChromePhp::log('instrumenty: ', $hodnoty, array_sum($hodnoty));

            $musicianCount = array_sum($hodnoty); // získání sumy zvoleného počtu muzikantů
            $type = 'int';
            settype($musicianCount, $type);


            //Pole služeb a její ceny
            $serviceStmt = $this->db->query("SELECT service_name, prices.cost AS profit, prices.price_from, prices.price_till 
                FROM services
                JOIN prices USING(service_id)
                WHERE CURDATE() BETWEEN prices.price_from AND prices.price_till");
            $service = $serviceStmt->fetchAll();
            $servicePrices = array();
            $chosenServices = array();
            foreach ($service as $s) {
                $servicePrices[$s['service_name']] = $s['profit'];
                $chosenServices[$s['service_name']] = true;
            }
           // ChromePhp::log('hodnoty sluzeb: ', $servicePrices);

            if (!$values['Conductor']) {
                $chosenServices['Conductor'] = false;
            }
            if (!$values['ClickTrack']) {
                $chosenServices['ClickTrack'] = false;
            }
            if (!$values['MusicPreparation']) {
                $chosenServices['MusicPreparation'] = false;
            }
            if (!$values['Orchestration']) {
                $chosenServices['Orchestration'] = false;
            }
            if (!$values['SoundEditing']) {
                $chosenServices['SoundEditing'] = false;
            }

            //získání ceny služby Conductor
            $stmt = $this->db->query("SELECT prices.cost AS profit"
                    . " FROM services "
                    . " JOIN prices USING (service_id)"
                    . " WHERE service_id = 1");
            $conductor = $stmt->fetch();

            //Cena za celkový počet hudebníků = Celkový počet muzikantů vynásobený cenou za 1 muzikanta. 
            $musicians = $musicianCount * $servicePrices['Musician'];

            //Ziskani ceny za orchestr
            $orchestra = ($musicians + $conductor['profit']) * $hours;

            //získání ceny za studio
            $studioTotal = $servicePrices['Studio'] * $hours;

            //získání ceny za sluzbu ClickTrack
            $clickTrack = $studioTotal * $servicePrices['ClickTrack'] / 100;

            //získání ceny za sluzbu Music Preparation
            $musicPreparation = $orchestra * $servicePrices['MusicPreparation'] / 100;

            //získání ceny za sluzbu Orchestration
            $orchestration = $orchestra * $servicePrices['Orchestration'] / 100;

            //získání ceny za sluzbu SoundEditing
            $soundEditing = $servicePrices['SoundEditing'] * $hours;

            //získání ceny bonusu
            $profit = $servicePrices['Profit'] * $hours;

            // Celková cena bez DPH 
            $subTotal = $orchestra + $studioTotal + $clickTrack + $musicPreparation + $orchestration + $soundEditing + $profit;

            //DPH 21%
            $vat = $subTotal * 0.21;
            // Celková cena včetně DPH 
            $totalSum = $subTotal + $vat;

            //Získání ceny z karty 
            $card = $totalSum * $servicePrices['Card'] / 100;

            //Celková cena s DPH i s kartou
            $totalSum += $card;
            
           $orderId = $this->createOrder($date, $userId, $chosenServices, $hours, $totalSum, $values['technology'], $hodnoty);
           return $orderId;
                
        } catch (Exception $ex) {
            $this->db->rollBack();
            return false;
        }
    }

    /** Vytvoření objednávky
     * @param date $term Datum objednaného nahrávání
     * @param int $userId ID uživatele
     * @param array $chosenServices Pole vybraných služeb
     * @param int $hours Počet hodin nahrávání
     * @param double $totalSum Celková cena objednávky s DPH
     * @param int $technologyId ID nahrávací technologie
     * @param array $hodnoty Pole hodnot instrumentů
     * @return int
     */
    private function createOrder($term, $userId, $chosenServices, $hours, $totalSum, $technologyId, $hodnoty) {
        try {
            $this->db->beginTransaction();
            $d = date("Y-m-d");
            $stav = "unprocessed";

            //Vytvoření nové objednávky
            $this->db->query("INSERT INTO orders(order_date, "
                    . "order_status, record_term, user_id,total_price, technology_id) "
                    . "VALUES (?, ?, ?, ?, ?, ?) ", $d, $stav, $term, $userId, $totalSum, $technologyId);

            //Poslední ID objednávky
            $lastId = $this->db->getInsertId();

            //Přidání služby do objednávky
            foreach ($chosenServices as $key => $value) {
                if ($value) {
                    $serviceId = $this->db->query("SELECT service_id FROM services "
                                    . "WHERE service_name = ?", $key)->fetch();
                    $this->db->query("INSERT INTO order_items(hours, order_id, service_id) "
                            . "VALUES(?,?,?)", $hours, $lastId, $serviceId['service_id']);
                }
            }

            //Přidání instrumentu do objednávky            
            foreach ($hodnoty as $key => $value) {
                if ($value > 0) {
                    $instrumentId = $this->db->query("SELECT instrument_id FROM instruments "
                                    . "WHERE name = ?", $key)->fetch();
                    $this->db->query("INSERT INTO order_instruments(order_id, instrument_id, counts) "
                            . "VALUES(?,?,?)", $lastId, $instrumentId['instrument_id'], $value);
                }
            }

            $this->db->commit();
            return $lastId;
            
        } catch (Exception $ex) {
            $this->db->rollBack();
            return false;
        }
    }

}
